
#include <avr/io.h>
#include "max7219.h"
#include <util/delay.h>
#include <stdlib.h>


void max7219_clock_out(unsigned char Data)
{
	char i;	
	for(i=0;i<8;i++) {	
		max7219_PORT &= ~(1<<max7219_CLK); //Clock auf Low
		if(Data & (1<<(7-i))) {
			max7219_PORT |= (1<<max7219_DIN);
		} else{
			max7219_PORT &= ~(1<<max7219_DIN);
		}
		max7219_PORT |= (1<<max7219_CLK); //Clock auf High, Daten werden bernommen
	}
	return;
}

void max7219_send(unsigned char Data, unsigned char Register)
{
	//Register &= 0x0F; //Obere 4 Bits der Adresse werden nciht gelesen.
	max7219_PORT &= ~(1<<max7219_LOAD); // LOAD auf Low (Chip aktivieren)
	max7219_clock_out(Register); //Adresse senden
	max7219_clock_out(Data); //Daten senden
	max7219_PORT |= (1<<max7219_LOAD); // LOAD auf High. Steigende Flanke an High bernimmt Daten in Adressiertes Register.
}

void max7219_set_decodemode(unsigned char Decodemode) {max7219_send(Decodemode, 0x09);}
void max7219_intensity(unsigned char Intensity) {max7219_send(Intensity, 0x0A);}
void max7219_set_scanlimit(unsigned char Scanlimit) {max7219_send(Scanlimit, 0x0B);} 

void max7219_shutdown(void) {max7219_send(0x00, 0x0C);} 
void max7219_wakeup(void) {max7219_send(0x01, 0x0C);} 

void max7219_displaytest_on(void) {max7219_send(0x01, 0x0F);} 
void max7219_displaytest_off(void) {max7219_send(0x00, 0x0F);} 

void max7219_nop(void) {max7219_send(0x00, 0x00);}



void max7219_clear(void)
{
	for(char i=1;i<9;i++){
		max7219_send(0b00000000, i);
	}
}

void max7219_init(void)
{
	max7219_DDR |= (1<<max7219_DIN) | (1<<max7219_LOAD) | (1<<max7219_CLK); // Die 3 Steurpins zu Ausgngen machen
	max7219_PORT &= ~( (1<<max7219_DIN) | (1<<max7219_CLK) ); // Clock und DIN auf Low Level
	max7219_PORT |= (1<<max7219_LOAD); //LOAD (CS bei 7221) auf High
	max7219_set_decodemode(0); //Keine BCD Wandler. Aller Segmente knnen individuell angesprochen werden.
	max7219_set_scanlimit(7); //Alle zeichen aktivieren
	max7219_intensity(7);	//Helligkeitsstufe 8 ist ein gutes Mittelma
	max7219_clear(); //Display zunchst leeren.
	max7219_wakeup();
}


void max7219_hex(unsigned char Number, unsigned char Position)
{
	unsigned char Segments;

		 if(Number == 0) {Segments = 0b01111110;} //0
	else if(Number == 1) {Segments = 0b00110000;} //1
	else if(Number == 2) {Segments = 0b01101101;} //2
	else if(Number == 3) {Segments = 0b01111001;} //3
	else if(Number == 4) {Segments = 0b00110011;} //4
	else if(Number == 5) {Segments = 0b01011011;} //5
	else if(Number == 6) {Segments = 0b01011111;} //6
	else if(Number == 7) {Segments = 0b01110000;} //7
	else if(Number == 8) {Segments = 0b01111111;} //8
	else if(Number == 9) {Segments = 0b01111011;} //9
	else if(Number == 10) {Segments = 0b01110111;} //A
	else if(Number == 11) {Segments = 0b00011111;} //b
	else if(Number == 12) {Segments = 0b01001110;} //C
	else if(Number == 13) {Segments = 0b00111101;} //d
	else if(Number == 14) {Segments = 0b01001111;} //E
	else if(Number == 15) {Segments = 0b01000111;} //F
		else 			{Segments = 0b00000000;} //Zeichen aus (Leezeichen)

	max7219_send(Segments, Position);
}



void max7219_disp_uint16(uint16_t Number)
{
	char Digits = 5; //Maximal 5 Stellen (65535)
		 if(Number<10)   {Digits = 1;}
	else if(Number<100)  {Digits = 2;}
	else if(Number<1000) {Digits = 3;}
	else if(Number<10000){Digits = 4;}
	
	char i;
	
	for(i=1;i<=Digits;i++)
	{	
		max7219_hex(Number%10,i);
		Number/=10;
	}
	for(;i<=5;i++){max7219_send(0b00000000, i);} //Fhrende Nullen nicht anzeigen

}



void max7219_disp_int16(int16_t Number)
{
	char Signed = 0;
	if(Number<0){
		Number = 0-Number;
		Signed = 1;
	}

	char Digits = 5; //Maximal 5 Stellen + Vorzeichen (32767...-32768)
		 if(Number<10)   {Digits = 1;}
	else if(Number<100)  {Digits = 2;}
	else if(Number<1000) {Digits = 3;}
	else if(Number<10000){Digits = 4;}
	
	char i;
	
	for(i=1;i<=Digits;i++)
	{	
		max7219_hex(Number%10,i);
		Number/=10;
	}
	if(Signed) {max7219_send(0b00000001, i); i++;} //Bei negativen Zahlen Minuszeichen davor
	for(;i<=6;i++){max7219_send(0b00000000, i);} //Fhrende Nullen nicht anzeigen

}


void max7219_disp_uint8(uint8_t Number)
{
	char Digits = 3; //Maximal 3 Stellen (255)
		 if(Number<10)   {Digits = 1;}
	else if(Number<100)  {Digits = 2;}
	
	char i;
	
	for(i=1;i<=Digits;i++)
	{	
		max7219_hex(Number%10,i);
		Number/=10;
	}
	for(;i<=3;i++){max7219_send(0b00000000, i);} //Fhrende Nullen nicht anzeigen

}


void max7219_disp_int8(int8_t Number)
{
	char Signed = 0;
	if(Number<0){
		Number = 0-Number;
		Signed = 1;
	}
	char Digits = 3;  //Maximal 3 Stellen + Vorzeichen ((127...-128)
		 if(Number<10)   {Digits = 1;}
	else if(Number<100)  {Digits = 2;}
	
	char i;
	
	for(i=1;i<=Digits;i++)
	{	
		max7219_hex(Number%10,i);
		Number/=10;
	}
	if(Signed) {max7219_send(0b00000001, i); i++;} //Bei negativen Zahlen Minuszeichen davor
	for(;i<=4;i++){max7219_send(0b00000000, i);} //Fhrende Nullen nicht anzeigen

}


